import * as React from 'react';
import { Box, Typography, TextField, Button, Select, MenuItem, Snackbar } from '@mui/material';
import QRCode from 'react-qr-code';
import {ContactPerson, Address, Dewar} from "../../openapi";


interface DewarDetailsProps {
    dewar: Dewar | null;
    trackingNumber: string;
    setTrackingNumber: React.Dispatch<React.SetStateAction<string>>;
    onGenerateQRCode: () => void;
    contactPersons: ContactPerson[];
    returnAddresses: Address[];
    addNewContactPerson: (name: string) => void;
    addNewReturnAddress: (address: string) => void;
    ready_date?: string;
    shipping_date?: string;
    arrival_date?: string;
}

const DewarDetails: React.FC<DewarDetailsProps> = ({
                                                       dewar,
                                                       trackingNumber,
                                                       setTrackingNumber,
                                                       onGenerateQRCode,
                                                       contactPersons,
                                                       returnAddresses,
                                                       addNewContactPerson,
                                                       addNewReturnAddress,
                                                   }) => {
    const [selectedContactPerson, setSelectedContactPerson] = React.useState<string>('');
    const [selectedReturnAddress, setSelectedReturnAddress] = React.useState<string>('');
    const [newContactPerson, setNewContactPerson] = React.useState<string>('');
    const [newReturnAddress, setNewReturnAddress] = React.useState<string>('');
    const [feedbackMessage, setFeedbackMessage] = React.useState<string>('');
    const [openSnackbar, setOpenSnackbar] = React.useState<boolean>(false);

    React.useEffect(() => {
        if (contactPersons.length > 0) {
            setSelectedContactPerson(contactPersons[0].firstname); // Default to the first contact person
        }
        if (returnAddresses.length > 0) {
            setSelectedReturnAddress(returnAddresses[0].return_address); // Default to the first return address
        }
    }, [contactPersons, returnAddresses]);

    if (!dewar) {
        return <Typography>No dewar selected.</Typography>;
    }

    const handleAddContact = () => {
        if (newContactPerson.trim() === '') {
            setFeedbackMessage('Please enter a valid contact person name.');
        } else {
            addNewContactPerson(newContactPerson);
            setNewContactPerson('');
            setFeedbackMessage('Contact person added successfully.');
        }
        setOpenSnackbar(true);
    };

    const handleAddAddress = () => {
        if (newReturnAddress.trim() === '') {
            setFeedbackMessage('Please enter a valid return address.');
        } else {
            addNewReturnAddress(newReturnAddress);
            setNewReturnAddress('');
            setFeedbackMessage('Return address added successfully.');
        }
        setOpenSnackbar(true);
    };

    return (
        <Box sx={{ marginTop: 2 }}>
            <Typography variant="h6">Selected Dewar: {dewar.dewar_name}</Typography>

            <TextField
                label="Tracking Number"
                value={trackingNumber}
                onChange={(e) => setTrackingNumber(e.target.value)}
                variant="outlined"
                sx={{ width: '300px', marginBottom: 2 }}
            />

            {/* QR Code display */}
            <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                <Box sx={{ width: 80, height: 80, backgroundColor: '#e0e0e0', display: 'flex', alignItems: 'center', justifyContent: 'center' }}>
                    {dewar.qrcode ? (
                        <QRCode value={dewar.qrcode} size={70} />
                    ) : (
                        <Typography>No QR code available</Typography>
                    )}
                </Box>
                <Button variant="contained" onClick={onGenerateQRCode}>
                    Generate QR Code
                </Button>
            </Box>

            <Typography variant="body1">Number of Pucks: {dewar.number_of_pucks}</Typography>
            <Typography variant="body1">Number of Samples: {dewar.number_of_samples}</Typography>

            {/* Dropdown for Contact Person */}
            <Typography variant="body1">Current Contact Person:</Typography>
            <Select
                value={selectedContactPerson}
                onChange={(e) => setSelectedContactPerson(e.target.value)}
                displayEmpty
                fullWidth
                sx={{ marginBottom: 2 }}
            >
                <MenuItem value="" disabled>Select Contact Person</MenuItem>
                {contactPersons.map((person) => (
                    <MenuItem key={person.id} value={person.firstname}>{person.lastname}</MenuItem>
                ))}
                <MenuItem value="add">Add New Contact Person</MenuItem>
            </Select>
            {selectedContactPerson === "add" && (
                <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                    <TextField
                        label="New Contact Person"
                        value={newContactPerson}
                        onChange={(e) => setNewContactPerson(e.target.value)}
                        variant="outlined"
                        sx={{ marginRight: 1, flexGrow: 1 }}
                    />
                    <Button variant="contained" onClick={handleAddContact}>
                        Add
                    </Button>
                </Box>
            )}

            {/* Dropdown for Return Address */}
            <Typography variant="body1">Current Return Address:</Typography>
            <Select
                value={selectedReturnAddress}
                onChange={(e) => setSelectedReturnAddress(e.target.value)}
                displayEmpty
                fullWidth
                sx={{ marginBottom: 2 }}
            >
                <MenuItem value="" disabled>Select Return Address</MenuItem>
                {returnAddresses.map((address) => (
                    <MenuItem key={address.id} value={address.address}>{address.address}</MenuItem>
                ))}
                <MenuItem value="add">Add New Return Address</MenuItem>
            </Select>
            {selectedReturnAddress === "add" && (
                <Box sx={{ display: 'flex', alignItems: 'center', marginBottom: 2 }}>
                    <TextField
                        label="New Return Address"
                        value={newReturnAddress}
                        onChange={(e) => setNewReturnAddress(e.target.value)}
                        variant="outlined"
                        sx={{ marginRight: 1, flexGrow: 1 }}
                    />
                    <Button variant="contained" onClick={handleAddAddress}>
                        Add
                    </Button>
                </Box>
            )}

            {/* Snackbar for feedback messages */}
            <Snackbar
                open={openSnackbar}
                autoHideDuration={6000}
                onClose={() => setOpenSnackbar(false)}
                message={feedbackMessage}
            />
        </Box>
    );
};

export default DewarDetails;
