import * as React from 'react';
import { Box, Button, TextField, Typography, Select, MenuItem, Stack, FormControl, InputLabel } from '@mui/material';
import { SelectChangeEvent } from '@mui/material';
import { SxProps } from '@mui/system';
import { ContactPerson, Address, Proposal, DefaultService, OpenAPI, Shipment_Input } from "../../openapi";
import { useEffect } from "react";

interface ShipmentFormProps {
    sx?: SxProps;
    onCancel: () => void;
}

const ShipmentForm: React.FC<ShipmentFormProps> = ({
                                                       sx = {},
                                                       onCancel,
                                                   }) => {
    const [contactPersons, setContactPersons] = React.useState<ContactPerson[]>([]);
    const [returnAddresses, setReturnAddresses] = React.useState<Address[]>([]);
    const [proposals, setProposals] = React.useState<Proposal[]>([]);
    const [isCreatingContactPerson, setIsCreatingContactPerson] = React.useState(false);
    const [isCreatingReturnAddress, setIsCreatingReturnAddress] = React.useState(false);

    const [newContactPerson, setNewContactPerson] = React.useState({
        firstName: '',
        lastName: '',
        phone_number: '',
        email: '',
    });

    const [newReturnAddress, setNewReturnAddress] = React.useState<Address>({
        street: '',
        city: '',
        zipcode: '',
        country: '',
    });

    const [newShipment, setNewShipment] = React.useState<Shipment_Input>({
        comments: undefined,
        contact_person: [],
        dewars: [],
        proposal_number: [],
        return_address: [],
        shipment_date: "",
        shipment_id: undefined,
        shipment_name: "",
        shipment_status: ""
    });

    const [errorMessage, setErrorMessage] = React.useState<string | null>(null);

    useEffect(() => {
        OpenAPI.BASE = 'http://127.0.0.1:8000'; // Define Base URL

        const getContacts = async () => {
            try {
                const c: ContactPerson[] = await DefaultService.getContactsContactsGet();
                setContactPersons(c);
            } catch (err) {
                setErrorMessage('Failed to load contact persons. Please try again later.');
            }
        };

        const getAddresses = async () => {
            try {
                const a: Address[] = await DefaultService.getReturnAddressesReturnAddressesGet();
                setReturnAddresses(a);
            } catch (err) {
                setErrorMessage('Failed to load return addresses. Please try again later.');
            }
        };

        const getProposals = async () => {
            try {
                const p: Proposal[] = await DefaultService.getProposalsProposalsGet();
                setProposals(p);
            } catch (err) {
                setErrorMessage('Failed to load proposals. Please try again later.');
            }
        };

        getContacts();
        getAddresses();
        getProposals();
    }, []);

    const validateEmail = (email: string) => /\S+@\S+\.\S+/.test(email);
    const validatePhoneNumber = (phone: string) => /^\+?[1-9]\d{1,14}$/.test(phone);
    const validateZipCode = (zipcode: string) => /^\d{5}(?:[-\s]\d{4})?$/.test(zipcode);

    const isContactFormValid = () => {
        const { firstName, lastName, phone_number, email } = newContactPerson;

        if (isCreatingContactPerson) {
            if (!firstName || !lastName || !validateEmail(email) || !validatePhoneNumber(phone_number)) return false;
        }

        return true;
    };

    const isAddressFormValid = () => {
        const { street, city, zipcode, country } = newReturnAddress;

        if (isCreatingReturnAddress) {
            if (!street || !city || !validateZipCode(zipcode) || !country) return false;
        }

        return true;
    };

    const isFormValid = () => {
        const { shipment_name, proposal_number, contact_person, return_address } = newShipment;

        if (!shipment_name || !proposal_number.length) return false;
        if (!contact_person.length || !return_address.length) return false;

        if (isCreatingContactPerson && !isContactFormValid()) return false;
        if (isCreatingReturnAddress && !isAddressFormValid()) return false;

        return true;
    };

    const handleChange = (e: React.ChangeEvent<HTMLInputElement>) => {
        const { name, value } = e.target;
        setNewShipment((prev) => ({ ...prev, [name]: value }));
        if (name === 'email') {
            setNewContactPerson((prev) => ({ ...prev, email: value }));
        }
        if (name === 'phone_number') {
            setNewContactPerson((prev) => ({ ...prev, phone_number: value }));
        }
        if (name === 'zipcode') {
            setNewReturnAddress((prev) => ({ ...prev, zipcode: value }));
        }
    };

    const handleSaveShipment = async () => {
        if (!isFormValid()) {
            setErrorMessage('Please fill in all mandatory fields correctly.');
            return;
        }

        OpenAPI.BASE = 'http://127.0.0.1:8000';

        const payload: Shipment_Input = {
            shipment_name: newShipment.shipment_name || '',
            shipment_date: new Date().toISOString().split('T')[0],
            shipment_status: 'In preparation',
            contact_person: newShipment.contact_person ? newShipment.contact_person.map(person => ({
                firstname: person.firstname,
                lastname: person.lastname,
                phone_number: person.phone_number,
                email: person.email
            })) : [],
            proposal_number: newShipment.proposal_number ? [{
                id: 1,
                number: newShipment.proposal_number
            }] : [],
            return_address: newShipment.return_address ? newShipment.return_address.map(address => ({
                street: address.street,
                city: address.city,
                zipcode: address.zipcode,
                country: address.country
            })) : [],
            comments: newShipment.comments || '',
            dewars: []
        };

        try {
            await DefaultService.createShipmentShipmentsPost(payload);
            setErrorMessage(null);
            // Handle successful save action
            onCancel(); // close the form after saving
        } catch (error) {
            setErrorMessage('Failed to save shipment. Please try again.');
        }
    };

    const handleContactPersonChange = (event: SelectChangeEvent) => {
        const value = event.target.value;
        if (value === 'new') {
            setIsCreatingContactPerson(true);
            setNewShipment({ ...newShipment, contact_person: [] });
        } else {
            setIsCreatingContactPerson(false);
            const selectedPerson = contactPersons.find((person) => person.lastname === value);
            if (selectedPerson) {
                setNewShipment({ ...newShipment, contact_person: [{ ...selectedPerson }] });
            }
        }
    };

    const handleReturnAddressChange = (event: SelectChangeEvent) => {
        const value = event.target.value;
        if (value === 'new') {
            setIsCreatingReturnAddress(true);
            setNewShipment({ ...newShipment, return_address: [] });
        } else {
            setIsCreatingReturnAddress(false);
            const selectedAddress = returnAddresses.find((address) => address.city === value);
            if (selectedAddress) {
                setNewShipment({ ...newShipment, return_address: [{ ...selectedAddress }] });
            }
        }
    };

    const handleProposalChange = (event: SelectChangeEvent) => {
        const selectedProposal = event.target.value;
        setNewShipment({ ...newShipment, proposal_number: selectedProposal });
    };

    const handleSaveNewContactPerson = async () => {
        if (!validateEmail(newContactPerson.email) || !validatePhoneNumber(newContactPerson.phone_number) ||
            !newContactPerson.firstName || !newContactPerson.lastName) {
            setErrorMessage('Please fill in all new contact person fields correctly.');
            return;
        }

        const payload = {
            firstname: newContactPerson.firstName,
            lastname: newContactPerson.lastName,
            phone_number: newContactPerson.phone_number,
            email: newContactPerson.email,
        };

        try {
            const c: ContactPerson = await DefaultService.createContactContactsPost(payload);
            setContactPersons([...contactPersons, c]);
            setErrorMessage(null);
        } catch (err) {
            setErrorMessage('Failed to create a new contact person. Please try again later.');
        }

        setNewContactPerson({ firstName: '', lastName: '', phone_number: '', email: '' });
        setIsCreatingContactPerson(false);
    };

    const handleSaveNewReturnAddress = async () => {
        if (!validateZipCode(newReturnAddress.zipcode) || !newReturnAddress.street || !newReturnAddress.city ||
            !newReturnAddress.country) {
            setErrorMessage('Please fill in all new return address fields correctly.');
            return;
        }

        const payload = {
            street: newReturnAddress.street.trim(),
            city: newReturnAddress.city.trim(),
            zipcode: newReturnAddress.zipcode.trim(),
            country: newReturnAddress.country.trim(),
        };

        try {
            const a: Address = await DefaultService.createReturnAddressReturnAddressesPost(payload);
            setReturnAddresses([...returnAddresses, a]);
            setErrorMessage(null);
        } catch (err) {
            setErrorMessage('Failed to create a new return address. Please try again later.');
        }

        setNewReturnAddress({ street: '', city: '', zipcode: '', country: '' });
        setIsCreatingReturnAddress(false);
    };

    return (
        <Box
            sx={{
                padding: 4,
                border: '1px solid #ccc',
                borderRadius: '4px',
                marginBottom: 2,
                maxWidth: '600px',
                ...sx,
            }}
        >
            <Typography variant="h6" sx={{ marginBottom: 2 }}>
                Create Shipment
            </Typography>
            {errorMessage && <Typography color="error">{errorMessage}</Typography>}
            <Stack spacing={2}>
                <TextField
                    label="Shipment Name"
                    name="shipment_name"
                    value={newShipment.shipment_name || ''}
                    onChange={handleChange}
                    fullWidth
                    required
                />
                <FormControl fullWidth required>
                    <InputLabel>Contact Person</InputLabel>
                    <Select
                        value={newShipment.contact_person?.[0]?.lastname || ''}
                        onChange={handleContactPersonChange}
                        displayEmpty
                    >
                        {contactPersons.map((person) => (
                            <MenuItem key={person.lastname + person.firstname} value={person.lastname}>
                                {`${person.lastname}, ${person.firstname}`}
                            </MenuItem>
                        ))}
                        <MenuItem value="new">
                            <em>Create New Contact Person</em>
                        </MenuItem>
                    </Select>
                </FormControl>
                {isCreatingContactPerson && (
                    <>
                        <TextField
                            label="First Name"
                            name="firstName"
                            value={newContactPerson.firstName}
                            onChange={(e) => setNewContactPerson({ ...newContactPerson, firstName: e.target.value })}
                            fullWidth
                            required
                        />
                        <TextField
                            label="Last Name"
                            name="lastName"
                            value={newContactPerson.lastName}
                            onChange={(e) => setNewContactPerson({ ...newContactPerson, lastName: e.target.value })}
                            fullWidth
                            required
                        />
                        <TextField
                            label="Phone"
                            name="phone_number"
                            type="tel"
                            value={newContactPerson.phone_number}
                            onChange={(e) => {
                                setNewContactPerson({ ...newContactPerson, phone_number: e.target.value });
                                handleChange(e);
                            }}
                            fullWidth
                            required
                            error={!validatePhoneNumber(newContactPerson.phone_number)}
                            helperText={!validatePhoneNumber(newContactPerson.phone_number) ? "Invalid phone number" : ""}
                        />
                        <TextField
                            label="Email"
                            name="email"
                            type="email"
                            value={newContactPerson.email}
                            onChange={(e) => {
                                setNewContactPerson({ ...newContactPerson, email: e.target.value });
                                handleChange(e);
                            }}
                            fullWidth
                            required
                            error={!validateEmail(newContactPerson.email)}
                            helperText={!validateEmail(newContactPerson.email) ? "Invalid email" : ""}
                        />
                        <Button variant="contained"
                                color="primary"
                                onClick={handleSaveNewContactPerson}
                                disabled={!isContactFormValid()}>
                            Save New Contact Person
                        </Button>
                    </>
                )}
                <FormControl fullWidth required>
                    <InputLabel>Proposal Number</InputLabel>
                    <Select
                        value={newShipment.proposal_number || ''}
                        onChange={handleProposalChange}
                        displayEmpty
                    >
                        {proposals.map((proposal) => (
                            <MenuItem key={proposal.id} value={proposal.number}>
                                {proposal.number}
                            </MenuItem>
                        ))}
                    </Select>
                </FormControl>
                <FormControl fullWidth required>
                    <InputLabel>Return Address</InputLabel>
                    <Select
                        value={newShipment.return_address?.[0]?.city || ''}
                        onChange={handleReturnAddressChange}
                        displayEmpty
                    >
                        {returnAddresses.map((address) => (
                            <MenuItem key={address.city} value={address.city}>
                                {`${address.street}, ${address.city}, ${address.zipcode}, ${address.country}`}
                            </MenuItem>
                        ))}
                        <MenuItem value="new">
                            <em>Create New Return Address</em>
                        </MenuItem>
                    </Select>
                </FormControl>
                {isCreatingReturnAddress && (
                    <>
                        <TextField
                            label="Street"
                            name="street"
                            value={newReturnAddress.street}
                            onChange={(e) => setNewReturnAddress({ ...newReturnAddress, street: e.target.value })}
                            fullWidth
                            required
                        />
                        <TextField
                            label="City"
                            name="city"
                            value={newReturnAddress.city}
                            onChange={(e) => setNewReturnAddress({ ...newReturnAddress, city: e.target.value })}
                            fullWidth
                            required
                        />
                        <TextField
                            label="Zip Code"
                            name="zipcode"
                            value={newReturnAddress.zipcode}
                            onChange={(e) => {
                                setNewReturnAddress({ ...newReturnAddress, zipcode: e.target.value });
                                handleChange(e);
                            }}
                            fullWidth
                            required
                            error={!validateZipCode(newReturnAddress.zipcode)}
                            helperText={!validateZipCode(newReturnAddress.zipcode) ? "Invalid zip code" : ""}
                        />
                        <TextField
                            label="Country"
                            name="country"
                            value={newReturnAddress.country}
                            onChange={(e) => setNewReturnAddress({ ...newReturnAddress, country: e.target.value })}
                            fullWidth
                            required
                        />
                        <Button variant="contained"
                                color="primary"
                                onClick={handleSaveNewReturnAddress}
                                disabled={!isAddressFormValid()}>
                            Save New Return Address
                        </Button>
                    </>
                )}
                <TextField
                    label="Comments"
                    name="comments"
                    fullWidth
                    multiline
                    rows={4}
                    value={newShipment.comments || ''}
                    onChange={handleChange}
                />
                <Stack direction="row" spacing={2} justifyContent="flex-end">
                    <Button variant="outlined" color="error" onClick={onCancel}>Cancel</Button>
                    <Button
                        variant="contained"
                        color="primary"
                        onClick={handleSaveShipment}
                        disabled={!isFormValid()}
                    >
                        Save Shipment
                    </Button>
                </Stack>
            </Stack>
        </Box>
    );
};

export default ShipmentForm;